@@ -3,25 +3,38 @@
 import sys
 from argparse import Namespace
 from pprint import pformat
-from typing import Any, NoReturn
+from typing import TYPE_CHECKING
 
 from .argparse import ArgumentParserWithUnknowns
 from .config import PikaurConfig
 from .i18n import translate, translate_many
 
+if TYPE_CHECKING:
+    from argparse import FileType
+    from collections.abc import Callable
+    from typing import Any, Final, NoReturn
+
 ArgSchema = list[tuple[str | None, str, None | bool | str | int]]
 PossibleArgValuesTypes = list[str] | str | bool | int | None
 
 
-def debug(msg: str | None = None) -> None:
+def print_stderr(msg: str | None = None) -> None:
     sys.stderr.write(f'{msg or ""}\n')
 
 
-def debug_format(msg: Any) -> None:
-    debug(pformat(msg))
+def pprint_stderr(msg: "Any") -> None:
+    print_stderr(pformat(msg))
 
 
-PACMAN_BOOL_OPTS: ArgSchema = [
+FLAG_READ_STDIN: "Final" = "-"
+
+
+class LiteralArgs:
+    NOCONFIRM: "Final" = "--noconfirm"
+    HELP: "Final" = "--help"
+
+
+PACMAN_BOOL_OPTS: "Final[ArgSchema]" = [
     # sync options
     ("S", "sync", None),
     ("g", "groups", None),
@@ -61,7 +74,7 @@ def get_pikaur_bool_opts() -> ArgSchema:
         (None, "keepbuilddeps", PikaurConfig().build.KeepBuildDeps.get_bool()),
         (None, "nodiff", PikaurConfig().review.NoDiff.get_bool()),
         (None, "rebuild", None),
-        (None, "dynamic-users", PikaurConfig().build.AlwaysUseDynamicUsers.get_bool()),
+        (None, "dynamic-users", PikaurConfig().build.DynamicUsers.get_str() == "always"),
         ("P", "pkgbuild", None),
         (None, "install", None),
         ("G", "getpkgbuild", None),
@@ -73,10 +86,11 @@ def get_pikaur_bool_opts() -> ArgSchema:
         (None, "skip-failed-build", PikaurConfig().build.SkipFailedBuild.get_bool()),
         # undocumented options:
         (None, "print-args-and-exit", None),
+        (None, "skip-aur-pull", None),
     ]
 
 
-PACMAN_STR_OPTS: ArgSchema = [
+PACMAN_STR_OPTS: "Final[ArgSchema]" = [
     (None, "color", None),
     ("b", "dbpath", None),  # @TODO: pyalpm?
     ("r", "root", None),
@@ -90,6 +104,11 @@ PACMAN_STR_OPTS: ArgSchema = [
 ]
 
 
+class ColorFlagValues:
+    ALWAYS: "Final" = "always"
+    NEVER: "Final" = "never"
+
+
 def get_pikaur_str_opts() -> ArgSchema:
     return [
         (None, "build-gpgdir", PikaurConfig().build.GpgDir.get_str()),
@@ -100,7 +119,13 @@ def get_pikaur_str_opts() -> ArgSchema:
     ]
 
 
-PACMAN_COUNT_OPTS: ArgSchema = [
+def get_pikaur_int_opts() -> ArgSchema:
+    return [
+        (None, "aur-clone-concurrency", None),
+    ]
+
+
+PACMAN_COUNT_OPTS: "Final[ArgSchema]" = [
     ("y", "refresh", 0),
     ("u", "sysupgrade", 0),
     ("c", "clean", 0),
@@ -115,7 +140,7 @@ def get_pikaur_count_opts() -> ArgSchema:
     ]
 
 
-PACMAN_APPEND_OPTS: ArgSchema = [
+PACMAN_APPEND_OPTS: "Final[ArgSchema]" = [
     (None, "ignore", None),
     (None, "ignoregroup", None),  # @TODO
     (None, "overwrite", None),
@@ -127,7 +152,8 @@ def get_pikaur_long_opts() -> list[str]:
     return [
         long_opt.replace("-", "_")
         for _short_opt, long_opt, _default in (
-            get_pikaur_bool_opts() + get_pikaur_str_opts() + get_pikaur_count_opts()
+            get_pikaur_bool_opts() + get_pikaur_str_opts()
+            + get_pikaur_count_opts() + get_pikaur_int_opts()
         )
     ]
 
@@ -169,15 +195,18 @@ class PikaurArgs(Namespace):
     config: str | None
     refresh: int
     clean: int
+    aur_clone_concurrency: int | None
+    skip_aur_pull: bool | None
     # positional: List[str]
     # @TODO: pylint bug:
     positional: list[str] = []
+    read_stdin: bool = False
 
     def __getattr__(self, name: str) -> PossibleArgValuesTypes:
         result: PossibleArgValuesTypes = getattr(
             super(),
             name,
-            getattr(self, name.replace("-", "_"))
+            getattr(self, name.replace("-", "_")),
         )
         return result
 
@@ -196,7 +225,7 @@ class PikaurArgs(Namespace):
             if self.check:  # handle "-k"
                 self.keepbuild = True
                 self.check = None
-        if self.sysupgrade:  # handle "-u"
+        if self.sysupgrade:  # handle "-u"  # noqa: SIM102
             if self.query:
                 self.sysupgrade = 0
                 self.upgrades = True
@@ -218,7 +247,7 @@ class PikaurArgs(Namespace):
 
     arg_depends = {
         "query": {
-            "upgrades": ["aur", "repo", ],
+            "upgrades": ["aur", "repo"],
         },
     }
 
@@ -237,12 +266,14 @@ class PikaurArgs(Namespace):
             cls,
             namespace: Namespace,
             unknown_args: list[str],
-            raw_args: list[str]
+            raw_args: list[str],
     ) -> "PikaurArgs":
         result = cls()
         for key, value in namespace.__dict__.items():
             setattr(result, key, value)
         result.unknown_args = unknown_args
+        if unknown_args and (result.pikaur_debug or result.debug):
+            print_stderr(translate("WARNING, unknown args: {}").format(unknown_args))
         result.raw = raw_args
         result.post_process_args()
         return result
@@ -258,7 +289,7 @@ class PikaurArgs(Namespace):
 
 class PikaurArgumentParser(ArgumentParserWithUnknowns):
 
-    def error(self, message: str) -> NoReturn:
+    def error(self, message: str) -> "NoReturn":
         exc = sys.exc_info()[1]
         if exc:
             raise exc
@@ -274,53 +305,66 @@ class PikaurArgumentParser(ArgumentParserWithUnknowns):
         return PikaurArgs.from_namespace(
             namespace=parsed_args,
             unknown_args=unknown_args,
-            raw_args=raw_args
+            raw_args=raw_args,
         )
 
-    def add_letter_andor_opt(
+    def add_letter_andor_opt(  # pylint: disable=too-many-arguments
             self,
             action: str | None = None,
             letter: str | None = None,
             opt: str | None = None,
-            default: PossibleArgValuesTypes = None
+            default: PossibleArgValuesTypes = None,
+            arg_type: "Callable[[str], Any] | FileType | None" = None,
     ) -> None:
         if action:
             if letter and opt:
                 self.add_argument(
-                    "-" + letter, "--" + opt, action=action, default=default
+                    "-" + letter, "--" + opt, action=action, default=default,
                 )
             elif opt:
                 self.add_argument(
-                    "--" + opt, action=action, default=default
+                    "--" + opt, action=action, default=default,
                 )
             elif letter:
                 self.add_argument(
-                    "-" + letter, action=action, default=default
+                    "-" + letter, action=action, default=default,
                 )
-        else:
+        elif arg_type:
             if letter and opt:
                 self.add_argument(
-                    "-" + letter, "--" + opt, default=default
+                    "-" + letter, "--" + opt, default=default, type=arg_type,
                 )
             elif opt:
                 self.add_argument(
-                    "--" + opt, default=default
+                    "--" + opt, default=default, type=arg_type,
                 )
             elif letter:
                 self.add_argument(
-                    "-" + letter, default=default
+                    "-" + letter, default=default, type=arg_type,
+                )
+        else:
+            if letter and opt:  # noqa: PLR5501
+                self.add_argument(
+                    "-" + letter, "--" + opt, default=default,
+                )
+            elif opt:
+                self.add_argument(
+                    "--" + opt, default=default,
+                )
+            elif letter:
+                self.add_argument(
+                    "-" + letter, default=default,
                 )
 
 
 class CachedArgs():
-
     args: PikaurArgs | None = None
 
 
-def debug_args(args: list[str], parsed_args: PikaurArgs) -> NoReturn:  # pragma: no cover
-    debug("Input:")
-    debug_format(args)
-    debug()
+def debug_args(args: list[str], parsed_args: PikaurArgs) -> "NoReturn":  # pragma: no cover
+    print_stderr("Input:")
+    pprint_stderr(args)
+    print_stderr()
     parsed_dict = vars(parsed_args)
     pikaur_long_opts = get_pikaur_long_opts()
     pacman_long_opts = get_pacman_long_opts()
@@ -334,20 +378,20 @@ def debug_args(args: list[str], parsed_args: PikaurArgs) -> NoReturn:  # pragma:
             pacman_dict[arg] = value
         else:
             misc_args[arg] = value
-    debug("PIKAUR parsed args:")
-    debug_format(pikaur_dict)
-    debug()
-    debug("PACMAN parsed args:")
-    debug_format(pacman_dict)
-    debug()
-    debug("MISC parsed args:")
-    debug_format(misc_args)
-    debug()
-    debug("Reconstructed pacman args:")
-    debug_format(reconstruct_args(parsed_args))
-    debug()
-    debug("Reconstructed pacman args without -S:")
-    debug_format(reconstruct_args(parsed_args, ignore_args=["sync"]))
+    print_stderr("PIKAUR parsed args:")
+    pprint_stderr(pikaur_dict)
+    print_stderr()
+    print_stderr("PACMAN parsed args:")
+    pprint_stderr(pacman_dict)
+    print_stderr()
+    print_stderr("MISC parsed args:")
+    pprint_stderr(misc_args)
+    print_stderr()
+    print_stderr("Reconstructed pacman args:")
+    pprint_stderr(reconstruct_args(parsed_args))
+    print_stderr()
+    print_stderr("Reconstructed pacman args without -S:")
+    pprint_stderr(reconstruct_args(parsed_args, ignore_args=["sync"]))
     sys.exit(0)
 
 
@@ -362,35 +406,40 @@ def parse_args(args: list[str] | None = None) -> PikaurArgs:
 
     for letter, opt, default in PACMAN_BOOL_OPTS + get_pikaur_bool_opts():
         parser.add_letter_andor_opt(
-            action="store_true", letter=letter, opt=opt, default=default
+            action="store_true", letter=letter, opt=opt, default=default,
         )
 
     for letter, opt, default in PACMAN_COUNT_OPTS + get_pikaur_count_opts():
         parser.add_letter_andor_opt(
-            action="count", letter=letter, opt=opt, default=default
+            action="count", letter=letter, opt=opt, default=default,
         )
 
     for letter, opt, default in PACMAN_APPEND_OPTS:
         parser.add_letter_andor_opt(
-            action="append", letter=letter, opt=opt, default=default
+            action="append", letter=letter, opt=opt, default=default,
         )
 
     for letter, opt, default in PACMAN_STR_OPTS + get_pikaur_str_opts():
         parser.add_letter_andor_opt(
-            action=None, letter=letter, opt=opt, default=default
+            action=None, letter=letter, opt=opt, default=default,
+        )
+
+    for letter, opt, default in get_pikaur_int_opts():
+        parser.add_letter_andor_opt(
+            action=None, letter=letter, opt=opt, default=default, arg_type=int,
         )
 
     parser.add_argument("positional", nargs="*")
 
     parsed_args = parser.parse_pikaur_args(args)
 
-    if parsed_args.positional and "-" in parsed_args.positional and not sys.stdin.isatty():
-        parsed_args.positional.remove("-")
-        parsed_args.positional += [
-            word
-            for line in sys.stdin.readlines()
-            for word in line.split()
-        ]
+    if (
+            parsed_args.positional
+            and FLAG_READ_STDIN in parsed_args.positional
+            and not sys.stdin.isatty()
+    ):
+        parsed_args.positional.remove(FLAG_READ_STDIN)
+        parsed_args.read_stdin = True
 
     if parsed_args.print_args_and_exit:  # pragma: no cover
         debug_args(args, parsed_args)
@@ -398,20 +447,20 @@ def parse_args(args: list[str] | None = None) -> PikaurArgs:
     try:
         parsed_args.validate()
     except IncompatibleArgumentsError as exc:
-        debug(translate(":: error: options {} can't be used together.").format(
-            ", ".join([f"'--{opt}'" for opt in exc.args])
+        print_stderr(translate(":: error: options {} can't be used together.").format(
+            ", ".join([f"'--{opt}'" for opt in exc.args]),
         ))
         sys.exit(1)
     except MissingArgumentError as exc:
-        debug(
+        print_stderr(
             translate_many(
                 ":: error: option {} can't be used without {}.",
                 ":: error: options {} can't be used without {}.",
-                len(exc.args[1:])
+                len(exc.args[1:]),
             ).format(
                 ", ".join([f"'--{opt}'" for opt in exc.args[1:]]),
-                f"'--{exc.args[0]}'"
-            )
+                f"'--{exc.args[0]}'",
+            ),
         )
         sys.exit(1)
     CachedArgs.args = parsed_args
@@ -422,7 +471,8 @@ def reconstruct_args(parsed_args: PikaurArgs, ignore_args: list[str] | None = No
     if not ignore_args:
         ignore_args = []
     for letter, opt, _default in (
-            get_pikaur_bool_opts() + get_pikaur_str_opts() + get_pikaur_count_opts()
+            get_pikaur_bool_opts() + get_pikaur_str_opts()
+            + get_pikaur_count_opts() + get_pikaur_int_opts()
     ):
         if letter:
             ignore_args.append(letter)
@@ -439,14 +489,14 @@ def reconstruct_args(parsed_args: PikaurArgs, ignore_args: list[str] | None = No
         for key, value in vars(parsed_args).items()
         if value
         if key not in ignore_args + count_args + [
-            "raw", "unknown_args", "positional",  # computed members
+            "raw", "unknown_args", "positional", "read_stdin",  # computed members
         ] + [
             long_arg
             for _short_arg, long_arg, default in PACMAN_STR_OPTS + PACMAN_APPEND_OPTS
         ]
     }
     result = list(set(
-        list(reconstructed_args.keys()) + parsed_args.unknown_args
+        list(reconstructed_args.keys()) + parsed_args.unknown_args,
     ))
     for args_key, value in vars(parsed_args).items():
         for letter, _opt, _default in PACMAN_COUNT_OPTS:
