@@ -2,14 +2,18 @@
 
 import configparser
 import os
+import random
 import sys
 from pathlib import Path
 from tempfile import gettempdir
-from typing import TYPE_CHECKING, Any, Callable, Final
+from typing import TYPE_CHECKING
 
-from .i18n import translate
+from .i18n import PIKAUR_NAME, translate
 
 if TYPE_CHECKING:
+    from collections.abc import Callable
+    from typing import Any, Final
+
     from typing_extensions import NotRequired, TypedDict
 
     class DeprecatedConfigValue(TypedDict):
@@ -24,246 +28,282 @@ if TYPE_CHECKING:
         migrated: NotRequired[bool]
 
 
-DEFAULT_CONFIG_ENCODING: Final = "utf-8"
+DEFAULT_CONFIG_ENCODING: "Final" = "utf-8"
+BOOL: "Final" = "bool"
+INT: "Final" = "int"
+STR: "Final" = "str"
+RUNNING_AS_ROOT: "Final" = os.geteuid() == 0  # @TODO: could global var be avoided here?
+VERSION: Final = "1.15-dev"
 
-RUNNING_AS_ROOT: Final = os.geteuid() == 0  # @TODO: could global var be avoided here?
-
-VERSION: Final = "${new_version}-dev"
-
-_USER_TEMP_ROOT: Final = gettempdir()
-_USER_CACHE_ROOT: Final = os.environ.get(
+_USER_TEMP_ROOT: "Final" = Path(gettempdir())
+_USER_CACHE_ROOT: "Final" = Path(os.environ.get(
     "XDG_CACHE_HOME",
-    os.path.join(Path.home(), ".cache/")
-)
+    Path.home() / ".cache/",
+))
 
-CACHE_ROOT: Final = (
-    "/var/cache/pikaur"
+CACHE_ROOT: "Final" = (
+    Path("/var/cache/pikaur")
     if RUNNING_AS_ROOT else
-    os.path.join(_USER_CACHE_ROOT, "pikaur/")
+    _USER_CACHE_ROOT / "pikaur/"
 )
 
-BUILD_CACHE_PATH: Final = os.path.join(CACHE_ROOT, "build")
-PACKAGE_CACHE_PATH: Final = os.path.join(CACHE_ROOT, "pkg")
+BUILD_CACHE_PATH: "Final" = CACHE_ROOT / "build"
+PACKAGE_CACHE_PATH: "Final" = CACHE_ROOT / "pkg"
 
-CONFIG_ROOT: Final = os.environ.get(
+CONFIG_ROOT: "Final" = Path(os.environ.get(
     "XDG_CONFIG_HOME",
-    os.path.join(Path.home(), ".config/")
-)
+    Path.home() / ".config/",
+))
 
-DATA_ROOT: Final = os.path.join(
-    os.environ.get(
+DATA_ROOT: "Final" = (
+    Path(os.environ.get(
         "XDG_DATA_HOME",
-        os.path.join(Path.home(), ".local/share/")
-    ), "pikaur"
+        Path.home() / ".local/share/",
+    )) / "pikaur"
 )
 
-_OLD_AUR_REPOS_CACHE_PATH: Final = os.path.join(CACHE_ROOT, "aur_repos")
-AUR_REPOS_CACHE_PATH: Final = (
-    os.path.join(CACHE_ROOT, "aur_repos")
+_OLD_AUR_REPOS_CACHE_PATH: "Final" = CACHE_ROOT / "aur_repos"
+AUR_REPOS_CACHE_PATH: "Final" = (
+    (CACHE_ROOT / "aur_repos")
     if RUNNING_AS_ROOT else
-    os.path.join(DATA_ROOT, "aur_repos")
+    (DATA_ROOT / "aur_repos")
 )
 
-BUILD_DEPS_LOCK: Final = os.path.join(
-    _USER_CACHE_ROOT if RUNNING_AS_ROOT else _USER_TEMP_ROOT,
-    "pikaur_build_deps.lock"
+BUILD_DEPS_LOCK: "Final" = (
+    (
+        _USER_CACHE_ROOT if RUNNING_AS_ROOT else _USER_TEMP_ROOT)
+    / "pikaur_build_deps.lock"
+)
+PROMPT_LOCK: "Final" = (
+    (
+        _USER_CACHE_ROOT if RUNNING_AS_ROOT else _USER_TEMP_ROOT
+    ) / f"pikaur_prompt_{random.randint(0, 999999)}.lock"  # nosec: B311   # noqa: S311
 )
 
 
-def get_config_path() -> str:
+def get_config_path() -> Path:
     config_flag = "--pikaur-config"
     if config_flag in sys.argv:
-        return sys.argv[
+        return Path(sys.argv[
             sys.argv.index(config_flag) + 1
-        ]
-    return os.path.join(
-        CONFIG_ROOT,
-        "pikaur.conf"
-    )
+        ])
+    return CONFIG_ROOT / "pikaur.conf"
+
+
+class UpgradeSortingValues:
+    VERSION: Final = "1.15-dev"
+    PKGNAME: "Final" = "pkgname"
+    REPO: "Final" = "repo"
+
+
+class AurSearchSortingValues:
+    HOTTEST: "Final" = "hottest"
+    PKGNAME: "Final" = "pkgname"
+    POPULARITY: "Final" = "popularity"
+    NUMVOTES: "Final" = "numvotes"
+    LASTMODIFIED: "Final" = "lastmodified"
+
+
+class DiffPagerValues:
+    AUTO: "Final" = "auto"
+    ALWAYS: "Final" = "always"
+    NEVER: "Final" = "never"
+
+
+CONFIG_YES_VALUES: "Final" = ("yes", "y", "true", "1")
 
 
 ConfigSchemaT = dict[str, dict[str, "ConfigValueType"]]
 CONFIG_SCHEMA: ConfigSchemaT = {
     "sync": {
         "AlwaysShowPkgOrigin": {
-            "data_type": "bool",
+            "data_type": BOOL,
             "default": "no",
         },
         "DevelPkgsExpiration": {
-            "data_type": "int",
+            "data_type": INT,
             "default": "-1",
         },
         "UpgradeSorting": {
-            "data_type": "str",
-            "default": "versiondiff"
+            "data_type": STR,
+            "default": UpgradeSortingValues.VERSIONDIFF,
         },
         "ShowDownloadSize": {
-            "data_type": "bool",
+            "data_type": BOOL,
             "default": "no",
         },
         "IgnoreOutofdateAURUpgrades": {
-            "data_type": "bool",
+            "data_type": BOOL,
             "default": "no",
         },
     },
     "build": {
         "KeepBuildDir": {
-            "data_type": "bool",
+            "data_type": BOOL,
             "default": "no",
         },
         "KeepDevBuildDir": {
-            "data_type": "bool",
+            "data_type": BOOL,
             "default": "yes",
         },
         "KeepBuildDeps": {
-            "data_type": "bool",
+            "data_type": BOOL,
             "default": "no",
         },
         "GpgDir": {
-            "data_type": "str",
-            "default": ("/etc/pacman.d/gnupg/" if RUNNING_AS_ROOT else "")
+            "data_type": STR,
+            "default": ("/etc/pacman.d/gnupg/" if RUNNING_AS_ROOT else ""),
         },
         "SkipFailedBuild": {
-            "data_type": "bool",
+            "data_type": BOOL,
             "default": "no",
         },
+        "DynamicUsers": {
+            "data_type": STR,
+            "default": "root",
+        },
         "AlwaysUseDynamicUsers": {
-            "data_type": "bool",
+            "data_type": BOOL,
             "default": "no",
+            "deprecated": {
+                "section": "build",
+                "option": "DynamicUsers",
+                "transform": lambda old_value, _config: "always" if old_value == "yes" else "root",
+            },
         },
         "NoEdit": {
-            "data_type": "bool",
+            "data_type": BOOL,
             "deprecated": {
                 "section": "review",
                 "option": "NoEdit",
             },
         },
         "DontEditByDefault": {
-            "data_type": "bool",
+            "data_type": BOOL,
             "deprecated": {
                 "section": "review",
                 "option": "DontEditByDefault",
             },
         },
         "NoDiff": {
-            "data_type": "bool",
+            "data_type": BOOL,
             "deprecated": {
                 "section": "review",
                 "option": "NoDiff",
             },
         },
         "GitDiffArgs": {
-            "data_type": "str",
+            "data_type": STR,
             "deprecated": {
                 "section": "review",
                 "option": "GitDiffArgs",
             },
         },
         "IgnoreArch": {
-            "data_type": "bool",
-            "default": "no"
+            "data_type": BOOL,
+            "default": "no",
         },
     },
     "review": {
         "NoEdit": {
-            "data_type": "bool",
+            "data_type": BOOL,
             "default": "no",
         },
         "DontEditByDefault": {
-            "data_type": "bool",
+            "data_type": BOOL,
             "default": "no",
         },
         "NoDiff": {
-            "data_type": "bool",
+            "data_type": BOOL,
             "default": "no",
         },
         "GitDiffArgs": {
-            "data_type": "str",
+            "data_type": STR,
             "default": "--ignore-space-change,--ignore-all-space",
         },
         "DiffPager": {
-            "data_type": "str",
-            "default": "auto"
+            "data_type": STR,
+            "default": DiffPagerValues.AUTO,
         },
         "HideDiffFiles": {
-            "data_type": "str",
-            "default": ".SRCINFO"
+            "data_type": STR,
+            "default": ".SRCINFO",
         },
     },
     "colors": {
         "Version": {
-            "data_type": "int",
+            "data_type": INT,
             "default": "10",
         },
         "VersionDiffOld": {
-            "data_type": "int",
+            "data_type": INT,
             "default": "11",
         },
         "VersionDiffNew": {
-            "data_type": "int",
+            "data_type": INT,
             "default": "9",
         },
     },
     "ui": {
         "RequireEnterConfirm": {
-            "data_type": "bool",
-            "default": "yes"
+            "data_type": BOOL,
+            "default": "yes",
         },
         "DiffPager": {
-            "data_type": "str",
+            "data_type": STR,
             "deprecated": {
                 "section": "review",
                 "option": "DiffPager",
             },
         },
         "PrintCommands": {
-            "data_type": "bool",
-            "default": "no"
+            "data_type": BOOL,
+            "default": "no",
         },
         "AurSearchSorting": {
-            "data_type": "str",
-            "default": "hottest"
+            "data_type": STR,
+            "default": AurSearchSortingValues.HOTTEST,
         },
         "DisplayLastUpdated": {
-            "data_type": "bool",
-            "default": "no"
+            "data_type": BOOL,
+            "default": "no",
         },
         "GroupByRepository": {
-            "data_type": "bool",
-            "default": "yes"
+            "data_type": BOOL,
+            "default": "yes",
         },
         "ReverseSearchSorting": {
-            "data_type": "bool",
-            "default": "no"
+            "data_type": BOOL,
+            "default": "no",
         },
         "WarnAboutPackageUpdates": {
-            "data_type": "str",
-            "default": ""
+            "data_type": STR,
+            "default": "",
         },
     },
     "misc": {
         "SudoLoopInterval": {
-            "data_type": "int",
+            "data_type": INT,
             "default": "59",
         },
         "PacmanPath": {
-            "data_type": "str",
-            "default": "pacman"
+            "data_type": STR,
+            "default": "pacman",
         },
         "PrivilegeEscalationTool": {
-            "data_type": "str",
+            "data_type": STR,
             "default": "sudo",
         },
         "AurHost": {
-            "data_type": "str",
+            "data_type": STR,
             "deprecated": {
                 "section": "network",
                 "option": "AurUrl",
-                "transform": lambda old_value, _config: f"https://{old_value}"
+                "transform": lambda old_value, _config: f"https://{old_value}",
             },
         },
         "NewsUrl": {
-            "data_type": "str",
+            "data_type": STR,
             "deprecated": {
                 "section": "network",
                 "option": "NewsUrl",
@@ -272,23 +312,23 @@ CONFIG_SCHEMA: ConfigSchemaT = {
     },
     "network": {
         "AurUrl": {
-            "data_type": "str",
+            "data_type": STR,
             "default": "https://aur.archlinux.org",
         },
         "NewsUrl": {
-            "data_type": "str",
+            "data_type": STR,
             "default": "https://www.archlinux.org/feeds/news/",
         },
         "Socks5Proxy": {
-            "data_type": "str",
+            "data_type": STR,
             "default": "",
         },
         "AurHttpProxy": {
-            "data_type": "str",
+            "data_type": STR,
             "default": "",
         },
         "AurHttpsProxy": {
-            "data_type": "str",
+            "data_type": STR,
             "default": "",
         },
     },
@@ -319,17 +359,14 @@ def write_config(config: configparser.ConfigParser | None = None) -> None:
                 config[section_name][option_name] = option_schema["default"]
                 need_write = True
     if need_write:
-        if not os.path.exists(CONFIG_ROOT):
-            os.makedirs(CONFIG_ROOT)
-        with open(get_config_path(), "w", encoding=DEFAULT_CONFIG_ENCODING) as configfile:
+        if not CONFIG_ROOT.exists():
+            CONFIG_ROOT.mkdir(parents=True)
+        with get_config_path().open("w", encoding=DEFAULT_CONFIG_ENCODING) as configfile:
             config.write(configfile)
 
 
 def str_to_bool(value: str) -> bool:
-    # pylint:disable=protected-access
-    result: bool = \
-        configparser.RawConfigParser()._convert_to_boolean(value)  # type: ignore[attr-defined]
-    return result
+    return value.lower() in CONFIG_YES_VALUES
 
 
 class PikaurConfigItem:
@@ -339,32 +376,32 @@ class PikaurConfigItem:
         self.key = key
         self.value = self.section.get(key)
         self._type_error_template = translate(
-            "{key} is not '{typeof}'"
+            "{key} is not '{typeof}'",
         )
 
     def get_bool(self) -> bool:
-        if get_key_type(self.section.name, self.key) != "bool":
-            not_bool_error = self._type_error_template.format(key=self.key, typeof="bool")
+        if get_key_type(self.section.name, self.key) != BOOL:
+            not_bool_error = self._type_error_template.format(key=self.key, typeof=BOOL)
             raise TypeError(not_bool_error)
         return str_to_bool(self.value)
 
     def get_int(self) -> int:
-        if get_key_type(self.section.name, self.key) != "int":
-            not_int_error = self._type_error_template.format(key=self.key, typeof="int")
+        if get_key_type(self.section.name, self.key) != INT:
+            not_int_error = self._type_error_template.format(key=self.key, typeof=INT)
             raise TypeError(not_int_error)
         return int(self.value)
 
     def get_str(self) -> str:
         # note: it"s basically needed for mypy
-        if get_key_type(self.section.name, self.key) != "str":
-            not_str_error = self._type_error_template.format(key=self.key, typeof="str")
+        if get_key_type(self.section.name, self.key) != STR:
+            not_str_error = self._type_error_template.format(key=self.key, typeof=STR)
             raise TypeError(not_str_error)
         return str(self.value)
 
     def __str__(self) -> str:
         return self.get_str()
 
-    def __eq__(self, item: Any) -> bool:
+    def __eq__(self, item: "Any") -> bool:
         result: bool = self.get_str() == item
         return result
 
@@ -392,7 +429,7 @@ class PikaurConfig():
         if not getattr(cls, "_config", None):
             config_path = get_config_path()
             cls._config = configparser.ConfigParser()
-            if not os.path.exists(config_path):
+            if not config_path.exists():
                 write_config()
             cls._config.read(config_path, encoding=DEFAULT_CONFIG_ENCODING)
             cls.migrate_config()
@@ -409,11 +446,12 @@ class PikaurConfig():
 
                 new_section_name: str = option_schema["deprecated"]["section"]
                 new_option_name: str = option_schema["deprecated"]["option"]
-                transform: Callable[
-                    [str, configparser.ConfigParser], str
-                ] | None = option_schema["deprecated"].get("transform")
+                transform: """Callable[
+                    [str, configparser.ConfigParser], str,
+                ] | None""" = option_schema["deprecated"].get("transform")
 
                 old_value_was_migrated = False
+                value_to_migrate = None
                 if (
                         new_section_name not in cls._config
                 ) or (
@@ -422,10 +460,12 @@ class PikaurConfig():
                     value_to_migrate = cls._config[section_name].get(option_name)
                     if value_to_migrate is not None:
                         if transform:
-                            value_to_migrate = transform(value_to_migrate, cls._config)
+                            new_value = transform(value_to_migrate, cls._config)
+                        else:
+                            new_value = value_to_migrate
                         if new_section_name not in cls._config:
                             cls._config[new_section_name] = {}
-                        cls._config[new_section_name][new_option_name] = value_to_migrate
+                        cls._config[new_section_name][new_option_name] = new_value
                         CONFIG_SCHEMA[new_section_name][new_option_name]["migrated"] = True
                         old_value_was_migrated = True
 
@@ -439,10 +479,11 @@ class PikaurConfig():
                     print(" ".join([  # noqa: T201
                         "::",
                         translate("warning:"),
-                        translate('Migrating [{}]{} config option to [{}]{} = "{}"...').format(
+                        translate('Migrating [{}]{}="{}" config option to [{}]{}="{}"...').format(
                             section_name, option_name,
+                            value_to_migrate or "",
                             new_section_name, new_option_name,
-                            cls._config[new_section_name][new_option_name]
+                            cls._config[new_section_name][new_option_name],
                         ),
                         "\n",
                     ]))
@@ -450,7 +491,7 @@ class PikaurConfig():
     @classmethod
     def validate_config(cls) -> None:
         pacman_path = cls._config["misc"]["PacmanPath"]
-        if pacman_path == "pikaur" or pacman_path.endswith("/pikaur"):
+        if pacman_path in (PIKAUR_NAME, sys.argv[0]) or pacman_path.endswith(f"/{PIKAUR_NAME}"):
             print("BAM! I am a shell bomb.")  # noqa: T201
             sys.exit(1)
 
