@@ -1,40 +1,44 @@
 """Licensed under GPLv3, see https://www.gnu.org/licenses/"""
 
 import os
-from typing import TypeVar
+from pathlib import Path
+from typing import TYPE_CHECKING
 
 from .args import parse_args
 from .config import _USER_TEMP_ROOT, CONFIG_ROOT
 from .core import open_file, running_as_root
 
+if TYPE_CHECKING:
+    from typing import Final, TypeVar
+
+    FallbackValueT = TypeVar("FallbackValueT")
+
 ConfigValueType = str | list[str] | None
 ConfigFormat = dict[str, ConfigValueType]
 
 
-FallbackValueT = TypeVar("FallbackValueT")
-
-
 class ConfigReader():
 
-    comment_prefixes = ("#", ";")
+    COMMENT_PREFIXES: "Final" = ("#", ";")
+    KEY_VALUE_DELIMITER: "Final" = "="
 
-    _cached_config: dict[str, ConfigFormat] | None = None
+    _cached_config: dict[str | Path, ConfigFormat] | None = None
     default_config_path: str
     list_fields: list[str] = []
     ignored_fields: list[str] = []
 
     @classmethod
     def _parse_line(cls, line: str) -> tuple[str | None, ConfigValueType]:
-        blank = (None, None, )
+        blank = (None, None)
         if line.startswith(" "):
             return blank
-        if "=" not in line:
+        if cls.KEY_VALUE_DELIMITER not in line:
             return blank
         line = line.strip()
-        for comment_prefix in cls.comment_prefixes:
+        for comment_prefix in cls.COMMENT_PREFIXES:
             line, *_comments = line.split(comment_prefix)
 
-        key, _sep, value = line.partition("=")
+        key, _sep, value = line.partition(cls.KEY_VALUE_DELIMITER)
         key = key.strip()
         value = value.strip()
 
@@ -53,7 +57,7 @@ class ConfigReader():
         return key, value
 
     @classmethod
-    def get_config(cls, config_path: str | None = None) -> ConfigFormat:
+    def get_config(cls, config_path: str | Path | None = None) -> ConfigFormat:
         config_path = config_path or cls.default_config_path
         if cls._cached_config is None:
             cls._cached_config = {}
@@ -74,40 +78,41 @@ class ConfigReader():
     def get(
             cls,
             key: str,
-            fallback: FallbackValueT | None = None,
-            config_path: str | None = None
-    ) -> ConfigValueType | FallbackValueT:
+            fallback: "FallbackValueT | None" = None,
+            config_path: str | Path | None = None,
+    ) -> "ConfigValueType | FallbackValueT":
         return cls.get_config(config_path=config_path).get(key) or fallback
 
 
 class MakepkgConfig():
 
-    _user_makepkg_path: str | None = "unset"
+    _UNSET: "Final" = object()
+    _user_makepkg_path: Path | object | None = _UNSET
 
     @classmethod
-    def get_user_makepkg_path(cls) -> str | None:
-        if cls._user_makepkg_path == "unset":
+    def get_user_makepkg_path(cls) -> Path | None:
+        if cls._user_makepkg_path is cls._UNSET:
             possible_paths = [
-                os.path.expanduser("~/.makepkg.conf"),
-                os.path.join(CONFIG_ROOT, "pacman/makepkg.conf"),
+                Path("~/.makepkg.conf").expanduser(),
+                CONFIG_ROOT / "pacman/makepkg.conf",
             ]
-            config_path: str | None = None
+            config_path: Path | None = None
             for path in possible_paths:
-                if os.path.exists(path):
+                if path.exists():
                     config_path = path
             cls._user_makepkg_path = config_path
-        return cls._user_makepkg_path
+        return cls._user_makepkg_path if isinstance(cls._user_makepkg_path, Path) else None
 
     @classmethod
     def get(
             cls,
             key: str,
-            fallback: FallbackValueT | None = None,
-            config_path: str | None = None
-    ) -> ConfigValueType | FallbackValueT:
+            fallback: "FallbackValueT | None" = None,
+            config_path: str | None = None,
+    ) -> "ConfigValueType | FallbackValueT":
         arg_path: str | None = parse_args().makepkg_config
-        value: ConfigValueType | FallbackValueT = ConfigReader.get(
-            key, fallback, config_path="/etc/makepkg.conf"
+        value: "ConfigValueType | FallbackValueT" = ConfigReader.get(
+            key, fallback, config_path="/etc/makepkg.conf",
         )
         if cls.get_user_makepkg_path():
             value = ConfigReader.get(key, value, config_path=cls.get_user_makepkg_path())
@@ -115,18 +120,17 @@ class MakepkgConfig():
             value = ConfigReader.get(key, value, config_path=arg_path)
         if config_path:
             value = ConfigReader.get(key, value, config_path=config_path)
-        return value  # noqa: RET504
+        return value
 
 
-def get_pkgdest() -> str | None:
+def get_pkgdest() -> Path | None:
     config_pkgdest = MakepkgConfig.get("PKGDEST")
     if not isinstance(config_pkgdest, str):
         config_pkgdest = None
     pkgdest: str | None = os.environ.get("PKGDEST", config_pkgdest)
-    if pkgdest:
-        pkgdest = pkgdest.replace("$HOME", "~")
-        pkgdest = os.path.expanduser(pkgdest)
-    return pkgdest  # noqa: RET504
+    if not pkgdest:
+        return None
+    return Path(pkgdest.replace("$HOME", "~")).expanduser()
 
 
 class MakePkgCommand:
@@ -136,15 +140,15 @@ class MakePkgCommand:
 
     @classmethod
     def _apply_dynamic_users_workaround(cls) -> None:
-        pkgdest = get_pkgdest()
+        pkgdest = str(get_pkgdest())
         if running_as_root() and pkgdest and (
-                pkgdest.startswith(_USER_TEMP_ROOT) or
-                pkgdest.startswith("/tmp") or  # nosec B108
-                pkgdest.startswith("/var/tmp")  # nosec B108
+                pkgdest.startswith(
+                    (str(_USER_TEMP_ROOT), "/tmp", "/var/tmp"),  # nosec B108  # noqa: S108
+                )
         ):
             if not cls._cmd:
-                raise RuntimeError()
-            cls._cmd = ["env", "PKGDEST="] + cls._cmd
+                raise RuntimeError
+            cls._cmd = ["env", "PKGDEST=", *cls._cmd]
             cls.pkgdest_skipped = True
 
     @classmethod
@@ -157,6 +161,6 @@ class MakePkgCommand:
             config_args = (
                 ["--config", args.makepkg_config] if args.makepkg_config else []
             )
-            cls._cmd = [args.makepkg_path or "makepkg", ] + makepkg_flags + config_args
+            cls._cmd = [args.makepkg_path or "makepkg", *makepkg_flags, *config_args]
             cls._apply_dynamic_users_workaround()
         return cls._cmd
