@@ -3,7 +3,7 @@
 import fnmatch
 import re
 from abc import ABCMeta, abstractmethod
-from typing import TYPE_CHECKING, Final, Pattern
+from typing import TYPE_CHECKING
 
 import pyalpm
 from pycman.config import PacmanConfig as PycmanConfig
@@ -14,17 +14,22 @@ from .core import DataType, PackageSource, spawn, sudo
 from .exceptions import DependencyError, PackagesNotFoundInRepoError
 from .i18n import translate
 from .lock import FancyLock
+from .logging import create_logger
 from .pacman_i18n import _p
-from .pprint import color_enabled, create_debug_logger, print_error, print_stderr
+from .pprint import color_enabled, print_error, print_stderr
 from .prompt import retry_interactive_command, retry_interactive_command_or_exit
 from .version import VersionMatcher
 
 if TYPE_CHECKING:
     # pylint: disable=cyclic-import
+    from pathlib import Path
+    from re import Pattern
+    from typing import Final
+
     from .aur import AURPackageInfo
 
 
-OFFICIAL_REPOS: Final = (
+OFFICIAL_REPOS: "Final" = (
     "core",
     "extra",
     "community",
@@ -35,22 +40,25 @@ OFFICIAL_REPOS: Final = (
 )
 
 
-_debug = create_debug_logger("pacman")
+REPO_NAME_DELIMITER: "Final" = "/"
 
 
-def create_pacman_pattern(pacman_message: str) -> Pattern[str]:
+logger = create_logger("pacman")
+
+
+def create_pacman_pattern(pacman_message: str) -> "Pattern[str]":
     return re.compile(
         _p(pacman_message).replace(
-            "(", r"\("
+            "(", r"\(",
         ).replace(
-            ")", r"\)"
+            ")", r"\)",
         ).replace(
-            "%d", "(.*)"
+            "%d", "(.*)",
         ).replace(
-            "%s", "(.*)"
+            "%s", "(.*)",
         ).replace(
-            "%zu", "(.*)"
-        ).strip()
+            "%zu", "(.*)",
+        ).strip(),
     )
 
 
@@ -58,14 +66,14 @@ def get_pacman_command(ignore_args: list[str] | None = None) -> list[str]:
     ignore_args = ignore_args or []
     args = parse_args()
     pacman_path = PikaurConfig().misc.PacmanPath.get_str()
-    pacman_cmd = [pacman_path, ]
+    pacman_cmd = [pacman_path]
     if color_enabled():
         pacman_cmd += ["--color=always"]
     else:
         pacman_cmd += ["--color=never"]
 
     for _short, arg, _default in PACMAN_STR_OPTS:
-        if arg in ["color", ]:  # we force it anyway
+        if arg in ["color"]:  # we force it anyway
             continue
         if arg in ignore_args:
             continue
@@ -74,7 +82,7 @@ def get_pacman_command(ignore_args: list[str] | None = None) -> list[str]:
             pacman_cmd += ["--" + arg, value]
 
     for _short, arg, _default in PACMAN_APPEND_OPTS:
-        if arg in ["ignore", ]:  # we reprocess it anyway
+        if arg in ["ignore"]:  # we reprocess it anyway
             continue
         if arg in ignore_args:
             continue
@@ -136,7 +144,7 @@ class PackageDBCommon(metaclass=ABCMeta):
 
     @classmethod
     def _discard_cache(
-            cls, package_source: PackageSource
+            cls, package_source: PackageSource,
     ) -> None:
         with get_db_lock(package_source)():
             if cls._packages_list_cache.get(package_source):
@@ -150,12 +158,12 @@ class PackageDBCommon(metaclass=ABCMeta):
 
     @classmethod
     def discard_local_cache(cls) -> None:
-        _debug("Discarding local cache...")
+        logger.debug("Discarding local cache...")
         cls._discard_cache(PackageSource.LOCAL)
 
     @classmethod
     def discard_repo_cache(cls) -> None:
-        _debug("Discarding repo cache...")
+        logger.debug("Discarding repo cache...")
         cls._discard_cache(PackageSource.REPO)
 
     @classmethod
@@ -197,8 +205,8 @@ class PackageDBCommon(metaclass=ABCMeta):
         return cls._packages_dict_cache[PackageSource.LOCAL]
 
     @classmethod
-    def _get_provided_dict(
-            cls, package_source: PackageSource
+    def get_provided_dict(
+            cls, package_source: PackageSource,
     ) -> dict[str, list[ProvidedDependency]]:
 
         if not cls._provided_dict_cache.get(package_source):
@@ -211,8 +219,8 @@ class PackageDBCommon(metaclass=ABCMeta):
                     ProvidedDependency(
                         name=pkg.name,
                         package=pkg,
-                        version_matcher=VersionMatcher(pkg.name, is_pkg_deps=True)
-                    )
+                        version_matcher=VersionMatcher(pkg.name, is_pkg_deps=True),
+                    ),
                 )
                 if pkg.provides:
                     for provided_pkg_line in pkg.provides:
@@ -222,8 +230,8 @@ class PackageDBCommon(metaclass=ABCMeta):
                             ProvidedDependency(
                                 name=provided_name,
                                 package=pkg,
-                                version_matcher=version_matcher
-                            )
+                                version_matcher=version_matcher,
+                            ),
                         )
             for what_provides, provided_pkgs in list(provided_pkg_names.items()):
                 if len(provided_pkgs) == 1 and provided_pkgs[0].name == what_provides:
@@ -233,11 +241,11 @@ class PackageDBCommon(metaclass=ABCMeta):
 
     @classmethod
     def get_repo_provided_dict(cls) -> dict[str, list[ProvidedDependency]]:
-        return cls._get_provided_dict(PackageSource.REPO)
+        return cls.get_provided_dict(PackageSource.REPO)
 
     @classmethod
     def get_local_provided_dict(cls) -> dict[str, list[ProvidedDependency]]:
-        return cls._get_provided_dict(PackageSource.LOCAL)
+        return cls.get_provided_dict(PackageSource.LOCAL)
 
     @classmethod
     def get_repo_pkgnames(cls) -> list[str]:
@@ -266,7 +274,7 @@ class PackageDB(PackageDBCommon):
             cls._alpm_handle = PacmanConfig().initialize_alpm()
         if not cls._alpm_handle:
             cant_init_alpm = translate("Cannot initialize ALPM")
-            raise Exception(cant_init_alpm)
+            raise RuntimeError(cant_init_alpm)
         return cls._alpm_handle
 
     @classmethod
@@ -305,16 +313,16 @@ class PackageDB(PackageDBCommon):
         return repos.index(repo_name)
 
     @classmethod
-    def _get_provided_dict(
-            cls, package_source: PackageSource
+    def get_provided_dict(
+            cls, package_source: PackageSource,
     ) -> dict[str, list[ProvidedDependency]]:
 
         if not cls._provided_dict_cache.get(package_source):
-            provided_pkg_names = super()._get_provided_dict(package_source)
+            provided_pkg_names = super().get_provided_dict(package_source)
             if package_source == PackageSource.REPO:
                 for _what_provides, provided_pkgs in provided_pkg_names.items():
                     provided_pkgs.sort(
-                        key=lambda p: f"{cls.get_repo_priority(p.package.db.name)}{p.package.name}"
+                        key=lambda p: f"{cls.get_repo_priority(p.package.db.name)}{p.package.name}",
                     )
             cls._provided_dict_cache[package_source] = provided_pkg_names
         return cls._provided_dict_cache[package_source]
@@ -328,8 +336,8 @@ class PackageDB(PackageDBCommon):
             names_only: bool = False,
             exact_match: bool = False,
     ) -> list[pyalpm.Package]:
-        if "/" in search_query:
-            db_name, search_query = search_query.split("/")
+        if REPO_NAME_DELIMITER in search_query:
+            db_name, search_query = search_query.split(REPO_NAME_DELIMITER)
         result = []
         for sync_db in reversed(cls.get_alpm_handle().get_syncdbs()):
             if not db_name or db_name == sync_db.name:
@@ -338,7 +346,7 @@ class PackageDB(PackageDBCommon):
                             (
                                 not names_only or search_query in pkg.name
                             ) and (
-                                not exact_match or search_query in ([pkg.name, ] + pkg.groups)
+                                not exact_match or search_query in ([pkg.name, *pkg.groups])
                             )
                     ):
                         result.append(pkg)
@@ -351,7 +359,7 @@ class PackageDB(PackageDBCommon):
                 if not quiet:
                     print_stderr(translate("Reading repository package databases..."))
                 cls._packages_list_cache[PackageSource.REPO] = cls.search_repo(
-                    search_query=""
+                    search_query="",
                 )
         return cls._packages_list_cache[PackageSource.REPO]
 
@@ -369,14 +377,14 @@ class PackageDB(PackageDBCommon):
 
     @classmethod
     def get_print_format_output(
-            cls, cmd_args: list[str], *, check_deps: bool = True, package_only: bool = False
+            cls, cmd_args: list[str], *, check_deps: bool = True, package_only: bool = False,
     ) -> list[PacmanPrint]:
         cache_index = " ".join(sorted(cmd_args))
         cached_pkg = cls._pacman_find_cache.get(cache_index)
         if cached_pkg is not None:
             return cached_pkg
         results: list[PacmanPrint] = []
-        final_args = cmd_args + ["--print-format", "%r/%n", ]
+        final_args = [*cmd_args, "--print-format", "%r/%n"]
         if not check_deps and not package_only:
             final_args.append("--nodeps")
         if package_only:
@@ -388,7 +396,7 @@ class PackageDB(PackageDBCommon):
         if found_packages_output:
             for line in found_packages_output.splitlines():
                 try:
-                    repo_name, pkg_name = line.split("/")
+                    repo_name, pkg_name = line.split(REPO_NAME_DELIMITER)
                 except ValueError:
                     print_stderr(line)
                     continue
@@ -396,7 +404,7 @@ class PackageDB(PackageDBCommon):
                     results.append(PacmanPrint(
                         full_name=line,
                         repo=repo_name,
-                        name=pkg_name
+                        name=pkg_name,
                     ))
         cls._pacman_find_cache[cache_index] = results
         return results
@@ -410,12 +418,12 @@ class PackageDB(PackageDBCommon):
         if cached_pkgs is not None:
             return cached_pkgs
         results: list[VersionMatcher] = []
-        not_found_packages_output = spawn(
+        not_found_packages_output = spawn([
             # pacman --deptest flag conflicts with some --sync options:
-            get_pacman_command(ignore_args=[
-                "overwrite"
-            ]) + ["--deptest", ] + cmd_args
-        ).stdout_text
+            *get_pacman_command(ignore_args=["overwrite"]),
+            "--deptest",
+            *cmd_args,
+        ]).stdout_text
         if not not_found_packages_output:
             cls._pacman_test_cache[cache_index] = results
             return results
@@ -439,24 +447,24 @@ class PackageDB(PackageDBCommon):
         not_found_pkg_names = []
         pkg_names_to_check: list[str] = []
         for pkg_name in pkg_lines:
-            if pkg_name in cls._pacman_repo_pkg_present_cache:
-                if not cls._pacman_repo_pkg_present_cache[pkg_name]:
-                    not_found_pkg_names.append(pkg_name)
-            else:
+            if pkg_name not in cls._pacman_repo_pkg_present_cache:
                 pkg_names_to_check += pkg_name.split(",")
+            elif not cls._pacman_repo_pkg_present_cache[pkg_name]:
+                not_found_pkg_names.append(pkg_name)
 
         if not pkg_names_to_check:
             return not_found_pkg_names
 
         results = (
-            spawn(
-                get_pacman_command() + ["--sync", "--print-format=%%", "--nodeps"] +
-                pkg_names_to_check
-            ).stderr_text or ""
+            spawn([
+                *get_pacman_command(),
+                "--sync", "--print-format=%%", "--nodeps",
+                *pkg_names_to_check,
+            ]).stderr_text or ""
         ).splitlines()
         new_not_found_pkg_names = []
         for result in results:
-            for pattern in (pattern_notfound, pattern_db_notfound, ):
+            for pattern in (pattern_notfound, pattern_db_notfound):
                 groups = pattern.findall(result)
                 if groups:
                     pkg_name = VersionMatcher(groups[0]).pkg_name
@@ -486,8 +494,8 @@ class PackageDB(PackageDBCommon):
         all_repo_pkgs = PackageDB.get_repo_dict()
         try:
             results = cls.get_print_format_output(
-                get_pacman_command() + ["--sync"] + pkg_name.split(","),
-                package_only=True
+                [*get_pacman_command(), "--sync", *pkg_name.split(",")],
+                package_only=True,
             )
         except DependencyError as exc:
             raise PackagesNotFoundInRepoError(packages=[pkg_name]) from exc
@@ -509,9 +517,9 @@ class PackageDB(PackageDBCommon):
 
 
 def get_upgradeable_package_names() -> list[str]:
-    upgradeable_packages_output = spawn(
-        get_pacman_command() + ["--query", "--upgrades", "--quiet"]
-    ).stdout_text
+    upgradeable_packages_output = spawn([
+        *get_pacman_command(), "--query", "--upgrades", "--quiet",
+    ]).stdout_text
     if not upgradeable_packages_output:
         return []
     return upgradeable_packages_output.splitlines()
@@ -527,18 +535,18 @@ def find_upgradeable_packages() -> list[pyalpm.Package]:
     all_local_pkgs = PackageDB.get_local_dict()
     results: list[PacmanPrint] = []
     try:
-        results = PackageDB.get_print_format_output(
-            get_pacman_command() + ["--sync"] + pkg_names
-        )
+        results = PackageDB.get_print_format_output([
+            *get_pacman_command(), "--sync", *pkg_names,
+        ])
     except DependencyError as exc:
         print_error(translate("Dependencies can't be satisfied for the following packages:"))
         print_stderr(" " * 12 + " ".join(pkg_names))
         print_stderr(str(exc))
         for pkg_name in pkg_names:
             try:
-                results += PackageDB.get_print_format_output(
-                    get_pacman_command() + ["--sync"] + [pkg_name, ]
-                )
+                results += PackageDB.get_print_format_output([
+                    *get_pacman_command(), "--sync", pkg_name,
+                ])
             except DependencyError as exc2:
                 print_error(translate("Because of:"))
                 print_stderr(str(exc2))
@@ -558,9 +566,9 @@ def find_sysupgrade_packages(ignore_pkgs: list[str] | None = None) -> list[pyalp
         extra_args.append(strip_repo_name(excluded_pkg_name))
 
     results = PackageDB.get_print_format_output(
-        get_pacman_command() + ["--sync"] +
+        [*get_pacman_command(), "--sync"] +
         ["--sysupgrade"] * parse_args().sysupgrade +
-        extra_args
+        extra_args,
     )
     return [
         all_repo_pkgs[result.full_name] for result in results
@@ -581,14 +589,14 @@ def refresh_pkg_db_if_needed() -> None:
     args = parse_args()
     if args.refresh:
         pacman_args = (sudo(
-            get_pacman_command() + ["--sync"] + ["--refresh"] * args.refresh
+            [*get_pacman_command(), "--sync"] + ["--refresh"] * args.refresh,
         ))
         retry_interactive_command_or_exit(pacman_args)
 
 
 def install_built_deps(
-        deps_names_and_paths: dict[str, str],
-        resolved_conflicts: list[list[str]] | None = None
+        deps_names_and_paths: "dict[str, Path]",
+        resolved_conflicts: list[list[str]] | None = None,
 ) -> None:
     if not deps_names_and_paths:
         return
@@ -609,18 +617,16 @@ def install_built_deps(
 
     explicitly_installed_deps = []
     for pkg_name, _path in deps_names_and_paths.items():
-        _debug(pkg_name)
+        logger.debug(pkg_name)
         if pkg_name in local_packages and local_packages[pkg_name].reason == 0:
             explicitly_installed_deps.append(pkg_name)
     deps_upgrade_success = True
     if len(explicitly_installed_deps) < len(deps_names_and_paths):
+        # @TODO: add support for --skip-failed-build here:
         deps_upgrade_success = retry_interactive_command(
             sudo(
-                _get_pacman_command() + [
-                    "--upgrade",
-                    "--asdeps",
-                ] + [
-                    path for name, path in deps_names_and_paths.items()
+                [*_get_pacman_command(), "--upgrade", "--asdeps"] + [
+                    str(path) for name, path in deps_names_and_paths.items()
                     if name not in explicitly_installed_deps
                 ],
             ),
@@ -632,12 +638,10 @@ def install_built_deps(
     if explicitly_installed_deps:
         explicit_upgrade_success = retry_interactive_command(
             sudo(
-                _get_pacman_command() + [
-                    "--upgrade",
-                ] + [
-                    path for name, path in deps_names_and_paths.items()
+                [*_get_pacman_command(), "--upgrade"] + [
+                    str(path) for name, path in deps_names_and_paths.items()
                     if name in explicitly_installed_deps
-                ]
+                ],
             ),
             pikspect=True,
             conflicts=resolved_conflicts,
@@ -646,16 +650,16 @@ def install_built_deps(
     PackageDB.discard_local_cache()
 
     if not (deps_upgrade_success and explicit_upgrade_success):
-        raise DependencyError()
+        raise DependencyError
 
 
 def strip_repo_name(pkg_name: str) -> str:
-    return pkg_name.split("/", 1)[-1]
+    return pkg_name.split(REPO_NAME_DELIMITER, 1)[-1]
 
 
 def get_ignored_pkgnames_from_patterns(
         orig_pkg_names: list[str],
-        ignore_patterns: list[str]
+        ignore_patterns: list[str],
 ) -> list[str]:
     ignored_pkg_names: list[str] = []
     for pkg_name in orig_pkg_names:
