@@ -1,5 +1,6 @@
 """Licensed under GPLv3, see https://www.gnu.org/licenses/"""
 
+import contextlib
 import fcntl
 import os
 import pty
@@ -13,44 +14,72 @@ import sys
 import termios
 import tty
 from multiprocessing.pool import ThreadPool
+from pathlib import Path
 from time import sleep
-from typing import Any, BinaryIO, Callable, Final, TextIO
+from typing import TYPE_CHECKING
 
 from .args import parse_args
+from .config import PikaurConfig
 from .core import DEFAULT_INPUT_ENCODING, get_sudo_refresh_command
 from .i18n import translate
+from .logging import create_logger
 from .pacman_i18n import _p
 from .pprint import (
     ColorsHighlight,
     PrintLock,
     bold_line,
     color_line,
-    create_debug_logger,
     get_term_width,
     print_stderr,
 )
 
-SMALL_TIMEOUT: Final = 0.01
+if TYPE_CHECKING:
+    from collections.abc import Callable
+    from typing import Any, BinaryIO, Final, TextIO
+
+
+SMALL_TIMEOUT: "Final" = 0.01
 
 
 TcAttrsType = list[int | list[bytes | int]]
 
 
-_debug = create_debug_logger("pikspect", lock=False)
+logger = create_logger("pikspect", lock=False)
+
+
+class ReadlineKeycodes:
+    CTRL_C: "Final" = 3
+    CTRL_D: "Final" = 4
+    CTRL_W: "Final" = 23
+    ENTER: "Final" = 13
+    BACKSPACE: "Final" = 127
 
 
 class TTYRestore():  # pragma: no cover
 
     old_tcattrs = None
+    old_tcattrs_out = None
+    old_tcattrs_err = None
     sub_tty_old_tcattrs = None
+    sub_tty_old_tcattrs_out = None
+    sub_tty_old_tcattrs_err = None
 
     @classmethod
     def save(cls) -> None:
         if sys.stdin.isatty():
             cls.old_tcattrs = termios.tcgetattr(sys.stdin.fileno())
+        if sys.stdout.isatty():
+            cls.old_tcattrs_out = termios.tcgetattr(sys.stdout.fileno())
+        if sys.stderr.isatty():
+            cls.old_tcattrs_err = termios.tcgetattr(sys.stderr.fileno())
 
     @classmethod
-    def _restore(cls, what: TcAttrsType | None = None) -> None:
+    def _restore(
+            cls,
+            what: TcAttrsType | None = None,
+            what_out: TcAttrsType | None = None,
+            what_err: TcAttrsType | None = None,
+    ) -> None:
         # if sys.stdout.isatty():
         #     termios.tcdrain(sys.stdout.fileno())
         # if sys.stderr.isatty():
@@ -61,20 +90,34 @@ class TTYRestore():  # pragma: no cover
             try:
                 termios.tcsetattr(sys.stdin.fileno(), termios.TCSANOW, what)
             except termios.error as exc:
-                _debug(",".join(str(arg) for arg in exc.args))
+                logger.debug(",".join(str(arg) for arg in exc.args))
+        if what_out:
+            try:
+                termios.tcsetattr(sys.stdout.fileno(), termios.TCSANOW, what_out)
+            except termios.error as exc:
+                logger.debug(",".join(str(arg) for arg in exc.args))
+        if what_err:
+            try:
+                termios.tcsetattr(sys.stderr.fileno(), termios.TCSANOW, what_err)
+            except termios.error as exc:
+                logger.debug(",".join(str(arg) for arg in exc.args))
 
     @classmethod
-    def restore(cls, *_whatever: Any) -> None:
-        cls._restore(cls.old_tcattrs)
+    def restore(cls, *_whatever: "Any") -> None:
+        cls._restore(cls.old_tcattrs, cls.old_tcattrs_out, cls.old_tcattrs_err)
 
     def __init__(self) -> None:
-        try:
+        with contextlib.suppress(termios.error, ValueError):
             self.sub_tty_old_tcattrs = termios.tcgetattr(sys.stdin.fileno())
-        except termios.error:
-            pass
+        with contextlib.suppress(termios.error):
+            self.sub_tty_old_tcattrs_out = termios.tcgetattr(sys.stdout.fileno())
+        with contextlib.suppress(termios.error):
+            self.sub_tty_old_tcattrs_err = termios.tcgetattr(sys.stderr.fileno())
 
-    def restore_new(self, *_whatever: Any) -> None:
-        self._restore(self.sub_tty_old_tcattrs)
+    def restore_new(self, *_whatever: "Any") -> None:
+        self._restore(
+            self.sub_tty_old_tcattrs, self.sub_tty_old_tcattrs_out, self.sub_tty_old_tcattrs_err,
+        )
 
 
 TTYRestore.save()
@@ -83,7 +126,7 @@ TTYRestore.save()
 def set_terminal_geometry(file_descriptor: int, rows: int, columns: int) -> None:
     term_geometry_struct = struct.pack("HHHH", rows, columns, 0, 0)
     fcntl.ioctl(
-        file_descriptor, termios.TIOCSWINSZ, term_geometry_struct
+        file_descriptor, termios.TIOCSWINSZ, term_geometry_struct,
     )
 
 
@@ -98,15 +141,15 @@ class TTYInputWrapper():  # pragma: no cover
         if self.is_pipe:
             self.old_stdin = sys.stdin
             try:
-                _debug("Attaching to TTY manually...")
-                sys.stdin = open("/dev/tty", encoding=DEFAULT_INPUT_ENCODING)
+                logger.debug("Attaching to TTY manually...")
+                sys.stdin = Path("/dev/tty").open(encoding=DEFAULT_INPUT_ENCODING)
                 self.tty_opened = True
             except Exception as exc:
-                _debug(exc)
+                logger.debug(exc)
 
-    def __exit__(self, *_exc_details: Any) -> None:
+    def __exit__(self, *_exc_details: "Any") -> None:
         if self.is_pipe and self.tty_opened:
-            _debug("Restoring stdin...", lock=False)
+            logger.debug("Restoring stdin...", lock=False)
             sys.stdin.close()
             sys.stdin = self.old_stdin
 
@@ -117,7 +160,7 @@ class NestedTerminal():
         self.tty_wrapper = TTYInputWrapper()
 
     def __enter__(self) -> os.terminal_size:
-        _debug("Opening virtual terminal...")
+        logger.debug("Opening virtual terminal...")
         self.tty_wrapper.__enter__()
         real_term_geometry = shutil.get_terminal_size((80, 80))
         for stream in (
@@ -129,25 +172,28 @@ class NestedTerminal():
                 tty.setcbreak(stream.fileno())
         return real_term_geometry
 
-    def __exit__(self, *exc_details: Any) -> None:
+    def __exit__(self, *exc_details: "Any") -> None:
         self.tty_wrapper.__exit__(*exc_details)
         TTYRestore.restore()
 
 
+RECOGNIZED_REGEX_SEQUENCES: "Final[tuple[str]]" = (".*", )
+
+
 def _match(pattern: str, line: str) -> bool:
     return len(line) >= len(pattern) and bool(
         re.compile(pattern).search(line)
-        if ".*" in pattern else
-        (pattern in line)
+        if max(sequence in pattern for sequence in RECOGNIZED_REGEX_SEQUENCES) else
+        (pattern in line),
     )
 
 
 class PikspectSignalHandler():
 
-    signal_handler: Callable[..., Any] | None = None
+    signal_handler: "Callable[..., Any] | None" = None
 
     @classmethod
-    def set_handler(cls, signal_handler: Callable[..., Any]) -> None:
+    def set_handler(cls, signal_handler: "Callable[..., Any]") -> None:
         cls.signal_handler = signal_handler
 
     @classmethod
@@ -155,7 +201,7 @@ class PikspectSignalHandler():
         cls.signal_handler = None
 
     @classmethod
-    def get(cls) -> Callable[..., Any] | None:
+    def get(cls) -> "Callable[..., Any] | None":
         return cls.signal_handler
 
 
@@ -165,8 +211,8 @@ class PikspectPopen(subprocess.Popen[bytes]):
     capture_input: bool
     capture_output: bool
     historic_output: list[bytes]
-    pty_in: TextIO
-    pty_out: BinaryIO
+    pty_in: "TextIO"
+    pty_out: "BinaryIO"
     default_questions: dict[str, list[str]]
     # max_question_length = 0  # preserve enough information to analyze questions
     max_question_length = get_term_width() * 2  # preserve also at least last line
@@ -182,7 +228,7 @@ class PikspectPopen(subprocess.Popen[bytes]):
             print_output: bool = True,
             capture_input: bool = True,
             capture_output: bool = False,
-            default_questions: dict[str, list[str]] | None = None
+            default_questions: dict[str, list[str]] | None = None,
     ) -> None:
         self.args = args
         self.print_output = print_output
@@ -222,41 +268,47 @@ class PikspectPopen(subprocess.Popen[bytes]):
     def run(self) -> None:
         if not isinstance(self.args, list):
             not_a_list_error = translate(
-                "`{var_name}` should be list."
+                "`{var_name}` should be list.",
             ).format(var_name="args")
             raise TypeError(not_a_list_error)
         PikspectSignalHandler.set_handler(
-            lambda *_whatever: self.send_signal(signal.SIGINT)
+            lambda *_whatever: self.send_signal(signal.SIGINT),
         )
         try:
             with NestedTerminal() as real_term_geometry:
 
-                if "sudo" in self.args:  # pragma: no cover
+                if (
+                        PikaurConfig().misc.PrivilegeEscalationTool.get_str() in self.args
+                ):  # pragma: no cover
                     subprocess.run(  # nosec B603
-                        get_sudo_refresh_command(),
-                        check=True
+                        get_sudo_refresh_command(),  # noqa: S603
+                        check=True,
                     )
-                with open(
-                        self.pty_user_master, "w", encoding=DEFAULT_INPUT_ENCODING
-                ) as self.pty_in:
-                    with open(self.pty_cmd_master, "rb", buffering=0) as self.pty_out:
-                        set_terminal_geometry(
-                            self.pty_out.fileno(),
-                            columns=real_term_geometry.columns,
-                            rows=real_term_geometry.lines
-                        )
-                        with ThreadPool(processes=3) as pool:
-                            output_task = pool.apply_async(self.cmd_output_reader_thread, ())
-                            input_task = pool.apply_async(self.user_input_reader_thread, ())
-                            communicate_task = pool.apply_async(self.communicator_thread, ())
-                            pool.close()
+                with (
+                        open(  # noqa: PTH123
+                            self.pty_user_master, "w", encoding=DEFAULT_INPUT_ENCODING,
+                        ) as self.pty_in,
+                        open(  # noqa: PTH123
+                            self.pty_cmd_master, "rb", buffering=0,
+                        ) as self.pty_out,
+                ):
+                    set_terminal_geometry(
+                        self.pty_out.fileno(),
+                        columns=real_term_geometry.columns,
+                        rows=real_term_geometry.lines,
+                    )
+                    with ThreadPool(processes=3) as pool:
+                        output_task = pool.apply_async(self.cmd_output_reader_thread, ())
+                        input_task = pool.apply_async(self.user_input_reader_thread, ())
+                        communicate_task = pool.apply_async(self.communicator_thread, ())
+                        pool.close()
 
-                            output_task.get()
-                            sys.stdout.buffer.write(self._write_buffer)
-                            sys.stdout.buffer.flush()
-                            communicate_task.get()
-                            input_task.get()
-                            pool.join()
+                        output_task.get()
+                        sys.stdout.buffer.write(self._write_buffer)
+                        sys.stdout.buffer.flush()
+                        communicate_task.get()
+                        input_task.get()
+                        pool.join()
         finally:
             PikspectSignalHandler.clear()
             os.close(self.pty_cmd_slave)
@@ -305,11 +357,10 @@ class PikspectPopen(subprocess.Popen[bytes]):
         while True:
 
             try:
-                selected = select.select([self.pty_out, ], [], [], SMALL_TIMEOUT)
+                selected = select.select([self.pty_out], [], [], SMALL_TIMEOUT)
             except ValueError:  # pragma: no cover
                 return
-            else:
-                readers = selected[0]
+            readers = selected[0]
             if not readers:
                 if self.returncode is not None:
                     break
@@ -321,7 +372,7 @@ class PikspectPopen(subprocess.Popen[bytes]):
             output = pty_reader.read(4096)
 
             self.historic_output = (
-                self.historic_output[-self.max_question_length:] + [output, ]
+                self.historic_output[-self.max_question_length:] + [output]
             )
             self.write_something(output)
             self.check_questions()
@@ -345,14 +396,14 @@ class PikspectPopen(subprocess.Popen[bytes]):
 
             try:
                 with PrintLock():
-                    if ord(char) == 127:  # BackSpace
+                    if ord(char) == ReadlineKeycodes.BACKSPACE:
                         sys.stdout.write("\b \b")
-                    if ord(char) == 23:  # Ctrl+W
+                    elif ord(char) == ReadlineKeycodes.CTRL_W:
                         sys.stdout.write(
                             "\r" + " " * get_term_width() + "\r" +
                             b"".join(
-                                self.historic_output
-                            ).decode(DEFAULT_INPUT_ENCODING).splitlines()[-1]
+                                self.historic_output,
+                            ).decode(DEFAULT_INPUT_ENCODING).splitlines()[-1],
                         )
                     self.pty_in.write(char)
                     self.pty_in.flush()
@@ -391,10 +442,10 @@ def pikspect(
             format_pacman_question("Do you want to remove these packages?"),
         ]
         CONFLICT = format_pacman_question(
-            "%s and %s are in conflict. Remove %s?", YesNo.QUESTION_YN_NO
+            "%s and %s are in conflict. Remove %s?", YesNo.QUESTION_YN_NO,
         )
         CONFLICT_VIA_PROVIDED = format_pacman_question(
-            "%s and %s are in conflict (%s). Remove %s?", YesNo.QUESTION_YN_NO
+            "%s and %s are in conflict (%s). Remove %s?", YesNo.QUESTION_YN_NO,
         )
 
     def format_conflicts(conflicts: list[list[str]]) -> list[str]:
@@ -404,7 +455,7 @@ def pikspect(
         ] + [
             (
                 re.escape(Questions.CONFLICT_VIA_PROVIDED % (
-                    new_pkg, old_pkg, ".*", old_pkg
+                    new_pkg, old_pkg, ".*", old_pkg,
                 ))
             ).replace(r"\.\*", ".*")
             for new_pkg, old_pkg in conflicts
@@ -435,7 +486,7 @@ def pikspect(
         if parse_args().print_commands:
             print_stderr(
                 color_line("pikspect => ", ColorsHighlight.cyan) +
-                " ".join(cmd)
+                " ".join(cmd),
             )
         proc.run()
         return proc
