@@ -1,14 +1,12 @@
 """Licensed under GPLv3, see https://www.gnu.org/licenses/"""
 
 from datetime import datetime
-from typing import Final, Sequence
-
-import pyalpm
+from typing import TYPE_CHECKING
 
 from .args import parse_args
-from .aur import AURPackageInfo, find_aur_packages
+from .aur import find_aur_packages
 from .config import PikaurConfig
-from .core import DEFAULT_TIMEZONE, AURInstallInfo, InstallInfo, RepoInstallInfo
+from .core import DEFAULT_TIMEZONE, AURInstallInfo, RepoInstallInfo
 from .exceptions import PackagesNotFoundInRepoError
 from .i18n import translate, translate_many
 from .pacman import (
@@ -24,9 +22,19 @@ from .print_department import (
     print_ignored_package,
     print_ignoring_outofdate_upgrade,
 )
-from .version import compare_versions
+from .version import VERSION_DEVEL, compare_versions
 
-DEVEL_PKGS_POSTFIXES: Final = (
+if TYPE_CHECKING:
+    from collections.abc import Sequence
+    from typing import Final
+
+    import pyalpm
+
+    from .aur import AURPackageInfo
+    from .core import InstallInfo
+
+
+DEVEL_PKGS_POSTFIXES: "Final" = (
     "-git",
     "-svn",
     "-bzr",
@@ -46,8 +54,8 @@ def is_devel_pkg(pkg_name: str) -> bool:
 
 
 def get_remote_package(
-        new_pkg_name: str
-) -> pyalpm.Package | AURPackageInfo | None:
+        new_pkg_name: str,
+) -> "pyalpm.Package | AURPackageInfo | None":
     try:
         repo_pkg = PackageDB.find_repo_package(new_pkg_name)
     except PackagesNotFoundInRepoError:
@@ -55,8 +63,7 @@ def get_remote_package(
         if aur_packages:
             return aur_packages[0]
         return None
-    else:
-        return repo_pkg
+    return repo_pkg
 
 
 def get_remote_package_version(new_pkg_name: str) -> str | None:
@@ -84,14 +91,14 @@ def find_repo_upgradeable() -> list[RepoInstallInfo]:
                 description=repo_pkg.desc,
                 repository=repo_pkg.db.name,
                 package=repo_pkg,
-            )
+            ),
         )
     return repo_packages_updates
 
 
 def find_aur_devel_updates(
-        aur_pkgs_info: list[AURPackageInfo],
-        package_ttl_days: int
+        aur_pkgs_info: "list[AURPackageInfo]",
+        package_ttl_days: int,
 ) -> list[AURInstallInfo]:
     local_packages = PackageDB.get_local_dict()
     now = datetime.now(tz=DEFAULT_TIMEZONE)
@@ -102,14 +109,14 @@ def find_aur_devel_updates(
             continue
         local_pkg = local_packages[pkg_name]
         pkg_install_datetime = datetime.fromtimestamp(
-            local_pkg.installdate, tz=DEFAULT_TIMEZONE
+            local_pkg.installdate, tz=DEFAULT_TIMEZONE,
         )
         pkg_age_days = (now - pkg_install_datetime).days
         if pkg_age_days >= package_ttl_days:
             aur_updates.append(AURInstallInfo(
                 name=pkg_name,
                 current_version=local_pkg.version,
-                new_version="devel",
+                new_version=VERSION_DEVEL,
                 description=aur_pkg.desc,
                 devel_pkg_age_days=pkg_age_days,
                 maintainer=aur_pkg.maintainer,
@@ -124,7 +131,7 @@ def find_aur_updates() -> tuple[list[AURInstallInfo], list[str]]:
     print_stderr(translate_many(
         "Reading AUR package info...",
         "Reading AUR packages info...",
-        len(package_names)
+        len(package_names),
     ))
     aur_pkgs_info, not_found_aur_pkgs = find_aur_packages(package_names)
     local_packages = PackageDB.get_local_dict()
@@ -158,18 +165,18 @@ def find_aur_updates() -> tuple[list[AURInstallInfo], list[str]]:
         if devel_packages_expiration > -1:
             aur_updates += find_aur_devel_updates(
                 aur_pkgs_up_to_date,
-                package_ttl_days=devel_packages_expiration
+                package_ttl_days=devel_packages_expiration,
             )
     return aur_updates, not_found_aur_pkgs
 
 
 def print_upgradeable(
-        aur_install_infos: Sequence[InstallInfo] | None = None,
+        aur_install_infos: "Sequence[InstallInfo] | None" = None,
         *,
         ignored_only: bool = False,
 ) -> None:
     args = parse_args()
-    updates: list[InstallInfo] = []
+    updates: "list[InstallInfo]" = []
     if aur_install_infos is not None:
         updates += aur_install_infos
     elif not args.repo:
@@ -182,11 +189,11 @@ def print_upgradeable(
     pkg_names = [pkg.name for pkg in updates]
     manually_ignored_pkg_names = get_ignored_pkgnames_from_patterns(
         pkg_names,
-        args.ignore
+        args.ignore,
     )
     config_ignored_pkg_names = get_ignored_pkgnames_from_patterns(
         pkg_names,
-        PacmanConfig().options.get("IgnorePkg", [])
+        PacmanConfig().options.get("IgnorePkg", []),
     )
     ignored_pkg_names = manually_ignored_pkg_names + config_ignored_pkg_names
     for pkg in updates[:]:
@@ -205,5 +212,5 @@ def print_upgradeable(
     else:
         print_stdout(pretty_format_upgradeable(
             updates,
-            print_repo=PikaurConfig().sync.AlwaysShowPkgOrigin.get_bool()
+            print_repo=PikaurConfig().sync.AlwaysShowPkgOrigin.get_bool(),
         ))
